require 'oauth2'

module Devbootcamp::OAuth
  class << self

    attr_accessor *%w(
      application_id
      secret
      site
      callback_url
      token
    )

    def client
      @client ||= OAuth2::Client.new(
        application_id, secret, site: site
      )
    end

    def site_uri
      URI(client.site)
    end

    def authorize_url(params={})
      params[:redirect_uri] ||= callback_url
      raise "redirect_uri cannot be blank" if params[:redirect_uri].blank?
      client.auth_code.authorize_url(params)
    end

    def unauthorize_url(params={})
      params[:redirect_uri] ||= callback_url
      raise "redirect_uri cannot be blank" if params[:redirect_uri].blank?
      site_uri.tap do |uri|
        uri.path = '/unauthenticate'
        uri.query = params.to_param
      end.to_s
    end

    def token_as_hash
      {
        token: token.token,
        refresh_token: token.refresh_token,
        expires_at: token.expires_at,
      }
    end

    def token_from_hash(hash)
      return unless hash
      @token = OAuth2::AccessToken.new(client, hash[:token], hash)
    end

    def authorized?
      !!token
    end

    def authorize!(oauth_code, options = {})
      @token = client.auth_code.get_token(oauth_code,
        :redirect_uri => options.fetch(:redirect_uri, callback_url)
      )
      authorized?
    end

    def refresh!
      self.token = token.refresh! if token && token.refresh_token
    end

  end

end
