module Devbootcamp::Rails

  def self.draw_routes(router)
    router.instance_eval do
      get 'sign_in'  => 'authentication#new',     as: 'sign_in'
      get 'sign_out' => 'authentication#destroy', as: 'sign_out'
      get 'authentication/oauth_callback' => 'authentication#oauth_callback', as: 'oauth_callback'
    end
  end

  def self.initialize!

    application_id, secret, site = ENV.values_at *%w{DBC_OAUTH_APPLICATION_ID DBC_OAUTH_SECRET DBC_OAUTH_SITE}

    if [application_id, secret, site].any?(&:blank?)
      abort <<-SH
ABORTING! None of these environment variables can be blank:
DBC_OAUTH_APPLICATION_ID=#{application_id}
DBC_OAUTH_SECRET=#{secret}
DBC_OAUTH_SITE=#{site}
      SH
    end

    Devbootcamp::OAuth.application_id = application_id
    Devbootcamp::OAuth.secret         = secret
    Devbootcamp::OAuth.site           = site

    if Rails.application.config.action_controller.perform_caching
      Devbootcamp::Auth.cache = Rails.cache
    end
  end

end
