module Devbootcamp::Rails::AuthenticationConcern
  extend ActiveSupport::Concern

  included do
    before_filter :set_oauth_token!
    before_filter :ensure_authenticated!
    helper_method :authenticated?
    helper_method :current_user
    # rescue_from OAuth2::Error, :with => :unauthenticate!
  end

  private

  def set_oauth_token!
    Devbootcamp::OAuth.callback_url = oauth_callback_url
    Devbootcamp::OAuth.token_from_hash(session[:oauth_token])
  end

  def ensure_authenticated!
    return if authenticated?
    redirect_to sign_in_path(r:request.url)
  end

  def authenticated?
    session[:oauth_token].is_a?(Hash)
  end

  def unauthenticate!
    session.clear
  end

  def current_user= current_user
    @current_user = current_user
    session[:current_user_attributes] = current_user.attributes
  end

  def current_user
    return nil unless authenticated?
    return @current_user if @current_user
    @current_user = Devbootcamp::User.new(session[:current_user_attributes])
  end

  def devbootcamp_oauth_authorize_url
    Devbootcamp::OAuth.authorize_url
  end

  def devbootcamp_oauth_unauthorize_url
    Devbootcamp::OAuth.unauthorize_url :redirect_uri => root_url
  end

end
